class Phaser(Module):
  __parameters__ = ["g1", "g2", "depth", "bias", ]
  __buffers__ = ["window_idx", "hann", "z", ]
  g1 : Tensor
  g2 : Tensor
  depth : Tensor
  bias : Tensor
  window_idx : Tensor
  hann : Tensor
  z : Tensor
  training : bool
  _is_full_backward_hook : Optional[bool]
  damped : bool
  sample_rate : int
  overlap : float
  window_size : int
  hop_size : int
  Nfft : int
  OLA_gain : float
  max_d : float
  min_d : float
  is_scriptable : bool
  lfo : __torch__.utils.DampedOscillator
  mlp : __torch__.utils.MLP
  bq : __torch__.utils.Biquad
  def forward(self: __torch__.phaser.Phaser,
    x: Tensor,
    sample_based: bool=True) -> Tuple[Tensor, Tensor]:
    device = ops.prim.device(x)
    x0 = torch.squeeze(x)
    sequence_length = (torch.size(x0))[0]
    hop_size = self.hop_size
    _0 = torch.floordiv(sequence_length, hop_size)
    num_hops = torch.add(_0, 1)
    _1 = torch.detach(torch.arange(0, num_hops))
    time = torch.to(torch.view(_1, [num_hops, 1]), device)
    lfo = self.lfo
    damped = self.damped
    lfo0 = (lfo).forward(annotate(int, time), damped, False, )
    mlp = self.mlp
    waveshaped_lfo = torch.squeeze((mlp).forward(lfo0, ))
    bias = self.bias
    depth = self.depth
    _2 = torch.mul(torch.mul(depth, 0.5), torch.add(waveshaped_lfo, 1))
    d = torch.add(bias, _2)
    _3 = torch.add(torch.neg(torch.tan(d)), 1.)
    _4 = torch.div(_3, torch.add(torch.tan(d), 1.))
    p = torch.tanh(_4)
    if sample_based:
      _6 = (self).forward_sample_based(x0, p, None, )
      y, _7, = _6
      _5 = (y, p)
    else:
      _8 = (self).forward_frame_based(x0, p, )
      _5 = (_8, p)
    return _5
  def forward_sample_based(self: __torch__.phaser.Phaser,
    x: Tensor,
    p: Tensor,
    zi: Optional[Tensor]=None) -> Tuple[Tensor, Tensor]:
    _9 = __torch__.utils.fourth_order_ap_coeffs
    _10 = __torch__.utils.linear_interpolate_dim
    _11 = __torch__.utils.sample_wise_lpc_scriptable
    if torch.eq(torch.dim(x), 1):
      x1 = torch.unsqueeze(x, 0)
    else:
      x1 = x
    if torch.eq(torch.dim(p), 1):
      p0 = torch.unsqueeze(p, 0)
    else:
      p0 = p
    sequence_length = (torch.size(x1))[-1]
    bq = self.bq
    DC = bq.DC
    bq0 = self.bq
    ff_params = bq0.ff_params
    b1 = torch.cat([DC, ff_params])
    bq1 = self.bq
    fb_params = bq1.fb_params
    a1 = __torch__.utils.logits2coeff(fb_params, )
    combine_a, combine_b, = _9(p0, )
    g2 = self.g2
    combine_denom = torch.sub(combine_a, torch.mul(torch.abs(g2), combine_b))
    _12 = torch.slice(combine_denom, -1, None, 1)
    combine_b0 = torch.div(combine_b, _12)
    _13 = torch.slice(combine_denom, -1, None, 1)
    combine_denom0 = torch.div(combine_denom, _13)
    _14 = torch.ne(sequence_length, (torch.size(p0))[-1])
    if _14:
      combine_b2 = _10(combine_b0, sequence_length, 1, True, )
      combine_denom2 = _10(combine_denom0, sequence_length, 1, True, )
      combine_denom1, combine_b1 = combine_denom2, combine_b2
    else:
      combine_denom1, combine_b1 = combine_denom0, combine_b0
    full_denom = __torch__.utils.combine_coeffs(a1, combine_denom1, )
    g1 = self.g1
    _15 = torch.add(torch.mul(g1, combine_denom1), combine_b1)
    full_b = __torch__.utils.combine_coeffs(b1, _15, )
    order = torch.sub(torch.size(full_b, -1), 1)
    if torch.__isnot__(zi, None):
      zi_a0 = unchecked_cast(Tensor, zi)
      zi_a : Optional[Tensor] = torch.flip(zi_a0, [-1])
    else:
      zi_a = zi
    y_a = _11(x1, torch.slice(full_denom, -1, 1), zi_a, )
    next_zi = torch.slice(y_a, -1, torch.neg(order))
    y_ab = __torch__.utils.time_varying_fir(y_a, full_b, zi, )
    return (y_ab, next_zi)
  def forward_frame_based(self: __torch__.phaser.Phaser,
    x: Tensor,
    p: Tensor) -> Tensor:
    _16 = __torch__.utils.linear_interpolate_dim
    Nfft = self.Nfft
    hop_size = self.hop_size
    window_size = self.window_size
    hann = self.hann
    X = __torch__.torch.functional.stft(x, Nfft, hop_size, window_size, hann, True, "constant", False, True, True, )
    n_frames = torch.size(X, 2)
    p1 = _16(p, n_frames, 1, True, )
    p2 = torch.unsqueeze(p1, 1)
    z = self.z
    z0 = torch.expand(torch.unsqueeze(z, 0), [2, -1, -1])
    _17 = torch.sub(p2, z0)
    _18 = torch.add(torch.neg(torch.mul(p2, z0)), 1)
    h_ap = torch.pow(torch.div(_17, _18), 4)
    bq = self.bq
    _19 = torch.view((bq).forward(), [-1, 1])
    g1 = self.g1
    g2 = self.g2
    _20 = torch.neg(torch.mul(torch.abs(g2), h_ap))
    _21 = torch.add(g1, torch.div(h_ap, torch.add(_20, 1)))
    h = torch.mul(_19, _21)
    Y = torch.mul(X, h)
    Nfft0 = self.Nfft
    window_size0 = self.window_size
    hop_size0 = self.hop_size
    hann0 = self.hann
    y = torch.istft(Y, Nfft0, hop_size0, window_size0, hann0, True, False, None, (torch.size(x))[-1])
    return y
