class DampedOscillator(Module):
  __parameters__ = ["sigma", "omega", "phi", "amp", ]
  __buffers__ = []
  sigma : Tensor
  omega : Tensor
  phi : Tensor
  amp : Tensor
  training : bool
  _is_full_backward_hook : Optional[bool]
  def forward(self: __torch__.utils.DampedOscillator,
    n: int,
    damped: bool,
    normalise: bool=False) -> Tensor:
    _0 = (self).get_r()
    omega = self.omega
    z = torch.polar(_0, omega)
    amp = self.amp
    phi = self.phi
    z0 = torch.polar(amp, phi)
    if torch.__not__(damped):
      z1 = torch.div(z, torch.abs(z))
    else:
      z1 = z
    if normalise:
      z02 = torch.div(z0, torch.abs(z0))
    else:
      z02 = z0
    _1 = torch.real(torch.mul(z02, torch.pow(z1, n)))
    return _1
  def get_r(self: __torch__.utils.DampedOscillator) -> Tensor:
    sigma = self.sigma
    _2 = torch.exp(torch.neg(torch.pow(sigma, 2)))
    return _2
class MLP(Module):
  __parameters__ = []
  __buffers__ = []
  training : bool
  _is_full_backward_hook : Optional[bool]
  model : __torch__.torch.nn.modules.container.Sequential
  def forward(self: __torch__.utils.MLP,
    x: Tensor) -> Tensor:
    model = self.model
    y = (model).forward(x, )
    return torch.view(y, torch.size(x))
class Biquad(Module):
  __parameters__ = ["ff_params", "fb_params", "DC", ]
  __buffers__ = ["pows", "z", "zpows", ]
  ff_params : Tensor
  fb_params : Tensor
  DC : Tensor
  pows : Tensor
  z : Tensor
  zpows : Tensor
  training : bool
  _is_full_backward_hook : Optional[bool]
  normalise : bool
  Nfft : int
  def forward(self: __torch__.utils.Biquad) -> Tensor:
    ff_params = self.ff_params
    zpows = self.zpows
    ff = torch.sum(torch.mul(ff_params, zpows), [1])
    normalise = self.normalise
    if normalise:
      ff0 = torch.add_(ff, 1.)
    else:
      DC = self.DC
      ff0 = torch.add_(ff, DC)
    fb_params = self.fb_params
    _3 = __torch__.utils.logits2coeff(fb_params, )
    _4 = torch.slice(torch.squeeze(_3), 0, 1)
    zpows0 = self.zpows
    _5 = torch.sum(torch.mul(_4, zpows0), [1])
    fb = torch.add(_5, 1.)
    return torch.div(ff0, fb)
def logits2coeff(logits: Tensor) -> Tensor:
  if torch.eq((torch.size(logits))[-1], 2):
    pass
  else:
    ops.prim.RaiseException("AssertionError: ")
  _6 = torch.tanh(torch.select(logits, -1, 0))
  a1 = torch.mul(_6, 2)
  a1_abs = torch.abs(a1)
  _7 = torch.add(torch.neg(a1_abs), 2)
  _8 = torch.tanh(torch.select(logits, -1, 1))
  a2 = torch.mul(torch.add(torch.mul(_7, _8), a1_abs), 0.5)
  _9 = torch.stack([torch.ones_like(a1), a1, a2], -1)
  return _9
def fourth_order_ap_coeffs(p: Tensor) -> Tuple[Tensor, Tensor]:
  _10 = [torch.pow(p, 4), torch.mul(torch.pow(p, 3), -4), torch.mul(torch.pow(p, 2), 6), torch.mul(p, -4), torch.ones_like(p)]
  b = torch.stack(_10, torch.dim(p))
  a = torch.flip(b, [-1])
  return (a, b)
def linear_interpolate_dim(x: Tensor,
    n: int,
    dim: int=-1,
    align_corners: bool=True) -> Tensor:
  _11 = __torch__.torch.nn.functional.interpolate
  n_dim = torch.dim(x)
  if torch.lt(0, n_dim):
    _12 = torch.le(n_dim, 3)
  else:
    _12 = False
  if _12:
    pass
  else:
    ops.prim.RaiseException("AssertionError: ")
  if torch.lt(dim, 0):
    dim0 = torch.add(n_dim, dim)
  else:
    dim0 = dim
  if torch.le(0, dim0):
    _13 = torch.lt(dim0, n_dim)
  else:
    _13 = False
  if _13:
    pass
  else:
    ops.prim.RaiseException("AssertionError: ")
  if torch.eq(torch.size(x, dim0), n):
    _14 = x
  else:
    if torch.eq(n_dim, 1):
      x0, swapped_dims = torch.view(x, [1, 1, -1]), False
    else:
      if torch.eq(n_dim, 2):
        if torch.ne(dim0, 0):
          pass
        else:
          ops.prim.RaiseException("AssertionError: ")
        x1, swapped_dims0 = torch.unsqueeze(x, 1), False
      else:
        if torch.eq(torch.dim(x), 3):
          if torch.ne(dim0, 0):
            pass
          else:
            ops.prim.RaiseException("AssertionError: ")
          if torch.eq(dim0, 1):
            x3, swapped_dims2 = torch.transpose(x, 1, 2), True
          else:
            x3, swapped_dims2 = x, False
          x2, swapped_dims1 = x3, swapped_dims2
        else:
          x2, swapped_dims1 = x, False
        x1, swapped_dims0 = x2, swapped_dims1
      x0, swapped_dims = x1, swapped_dims0
    x4 = _11(x0, n, None, "linear", align_corners, None, False, )
    if torch.eq(n_dim, 1):
      x5 = torch.view(x4, [-1])
    else:
      if torch.eq(n_dim, 2):
        x6 = torch.squeeze(x4, 1)
      else:
        if swapped_dims:
          x7 = torch.transpose(x4, 1, 2)
        else:
          x7 = x4
        x6 = x7
      x5 = x6
    _14 = x5
  return _14
def combine_coeffs(x: Tensor,
    y: Tensor) -> Tensor:
  pad_size = torch.sub(torch.size(x, -1), 1)
  y0 = torch.unfold(torch.pad(y, [pad_size, pad_size]), -1, torch.size(x, -1), 1)
  _15 = torch.unsqueeze(torch.flip(x, [-1]), -1)
  _16 = torch.squeeze(torch.matmul(y0, _15), -1)
  return _16
def sample_wise_lpc_scriptable(x: Tensor,
    a: Tensor,
    zi: Optional[Tensor]=None) -> Tensor:
  if torch.eq(torch.dim(x), 2):
    pass
  else:
    ops.prim.RaiseException("AssertionError: ")
  if torch.eq(torch.dim(a), 3):
    pass
  else:
    ops.prim.RaiseException("AssertionError: ")
  _17 = torch.eq(torch.size(x, 0), torch.size(a, 0))
  if _17:
    pass
  else:
    ops.prim.RaiseException("AssertionError: ")
  _18 = torch.eq(torch.size(x, 1), torch.size(a, 1))
  if _18:
    pass
  else:
    ops.prim.RaiseException("AssertionError: ")
  B, T, order, = torch.size(a)
  if torch.__is__(zi, None):
    zi0 = torch.new_zeros(a, [B, order])
  else:
    zi1 = unchecked_cast(Tensor, zi)
    _19 = torch.eq(torch.size(zi1), [B, order])
    if _19:
      pass
    else:
      ops.prim.RaiseException("AssertionError: ")
    zi0 = zi1
  _20 = torch.add(T, order)
  _21 = ops.prim.dtype(x)
  padded_y = torch.empty([B, _20], dtype=_21)
  zi2 = torch.flip(zi0, [1])
  _22 = torch.slice(torch.slice(padded_y), 1, None, order)
  _23 = torch.copy_(_22, zi2)
  _24 = torch.slice(torch.slice(padded_y), 1, order)
  _25 = torch.copy_(_24, x)
  a_flip = torch.flip(a, [2])
  for t in range(T):
    _26 = torch.add(t, order)
    _27 = torch.select(torch.slice(padded_y), 1, _26)
    _28 = torch.slice(torch.slice(a_flip), 1, t, torch.add(t, 1))
    _29 = torch.slice(torch.slice(padded_y), 1, t, torch.add(t, order))
    _30 = torch.matmul(_28, torch.unsqueeze(_29, 2))
    _31 = torch.select(torch.slice(_30), 1, 0)
    _32 = torch.sub_(_27, torch.select(_31, 1, 0))
  _33 = torch.slice(torch.slice(padded_y), 1, order)
  return _33
def time_varying_fir(x: Tensor,
    b: Tensor,
    zi: Optional[Tensor]=None) -> Tensor:
  if torch.eq(torch.dim(x), 2):
    pass
  else:
    ops.prim.RaiseException("AssertionError: ")
  if torch.eq(torch.dim(b), 3):
    pass
  else:
    ops.prim.RaiseException("AssertionError: ")
  _34 = torch.eq(torch.size(x, 0), torch.size(b, 0))
  if _34:
    pass
  else:
    ops.prim.RaiseException("AssertionError: ")
  _35 = torch.eq(torch.size(x, 1), torch.size(b, 1))
  if _35:
    pass
  else:
    ops.prim.RaiseException("AssertionError: ")
  order = torch.sub(torch.size(b, 2), 1)
  x_padded = torch.pad(x, [order, 0])
  if torch.__isnot__(zi, None):
    zi3 = unchecked_cast(Tensor, zi)
    _36 = torch.eq(torch.size(zi3), [torch.size(x, 0), order])
    if _36:
      pass
    else:
      ops.prim.RaiseException("AssertionError: ")
    _37 = torch.slice(torch.slice(x_padded), 1, None, order)
    _38 = torch.copy_(_37, zi3)
  else:
    pass
  x_unfolded = torch.unfold(x_padded, 1, torch.add(order, 1), 1)
  x_unfolded0 = torch.unsqueeze(x_unfolded, 3)
  b0 = torch.unsqueeze(torch.flip(b, [2]), 2)
  y = torch.matmul(b0, x_unfolded0)
  y1 = torch.squeeze(y, 3)
  return torch.squeeze(y1, 2)
